#ifndef _PLYCONVERTER_H
#define _PLYCONVERTER_H

#include "SimpleVec.h"
#include <string>
#include <vector>
#include <map>


static std::string model_path("data/city/model/");


typedef std::vector<SimpleVec> VertexArray;
typedef std::pair<float, float> Texcoord;
typedef std::vector<SimpleVec> TexcoordArray;
typedef std::map<std::string, int> TextureTable;
typedef std::vector<std::string> TextureArray;


class ogzstream;



/** Converts obj format into objects readable by our format
*/
class PlyConverter
{
public:

	~PlyConverter();

	bool Convert(const std::string &inputFilename, 
		         const std::string &outputFilename);
	             // const std::string textureFilename) const;


protected:

	void LoadShape(const VertexArray &faceVertices,
		           const VertexArray &faceNormals,
				   const std::vector<Texcoord> &faceTexcoords
				   );

	struct Geometry
	{
		VertexArray mVertices;
		VertexArray mNormals;
		TexcoordArray mTexcoords;
	};

	typedef std::vector<Geometry *> GeometryArray;

	void WriteGeometry(ogzstream &str, Geometry *geom);

	bool ReadFile(const std::string &inputFilename);
	bool WriteFile(const std::string &outputFilename);


	TextureTable mTextureTable;
	
	TextureArray mTextures;
	GeometryArray mGeometry;

	int mNumShapes;
};



#endif
